@@ -6,7 +6,7 @@ Refactored MarkdownFlow class with built-in LLM processing capabilities and unif
 
 import json
 import re
-from collections.abc import AsyncGenerator
+from collections.abc import Generator
 from copy import copy
 from typing import Any
 
@@ -176,14 +176,15 @@ class MarkdownFlow:
 
     # Core unified interface
 
-    async def process(
+    def process(
         self,
         block_index: int,
         mode: ProcessMode = ProcessMode.COMPLETE,
         context: list[dict[str, str]] | None = None,
         variables: dict[str, str | list[str]] | None = None,
         user_input: dict[str, list[str]] | None = None,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+        dynamic_interaction_format: str | None = None,
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """
         Unified block processing interface.
 
@@ -193,9 +194,10 @@ class MarkdownFlow:
             context: Context message list
             variables: Variable mappings
             user_input: User input (for interaction blocks)
+            dynamic_interaction_format: Dynamic interaction format for validation
 
         Returns:
-            LLMResult or AsyncGenerator[LLMResult, None]
+            LLMResult or Generator[LLMResult, None, None]
         """
         # Process document_prompt variable replacement
         if self._document_prompt:
@@ -204,56 +206,70 @@ class MarkdownFlow:
         block = self.get_block(block_index)
 
         if block.block_type == BlockType.CONTENT:
-            return await self._process_content(block_index, mode, context, variables)
+            # Check if this is dynamic interaction validation
+            if dynamic_interaction_format and user_input:
+                return self._process_dynamic_interaction_validation(block_index, dynamic_interaction_format, user_input, mode, context, variables)
+            # Normal content processing (possibly with dynamic conversion)
+            return self._process_content(block_index, mode, context, variables)
 
         if block.block_type == BlockType.INTERACTION:
             if user_input is None:
                 # Render interaction content
-                return await self._process_interaction_render(block_index, mode, variables)
+                return self._process_interaction_render(block_index, mode, variables)
             # Process user input
-            return await self._process_interaction_input(block_index, user_input, mode, context, variables)
+            return self._process_interaction_input(block_index, user_input, mode, context, variables)
 
         if block.block_type == BlockType.PRESERVED_CONTENT:
             # Preserved content output as-is, no LLM call
-            return await self._process_preserved_content(block_index, variables)
+            return self._process_preserved_content(block_index, variables)
 
         # Handle other types as content
-        return await self._process_content(block_index, mode, context, variables)
+        return self._process_content(block_index, mode, context, variables)
 
     # Internal processing methods
 
-    async def _process_content(
+    def _process_content(
         self,
         block_index: int,
         mode: ProcessMode,
         context: list[dict[str, str]] | None,
         variables: dict[str, str | list[str]] | None,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """Process content block."""
-        # Build messages
-        messages = self._build_content_messages(block_index, variables)
 
+        # For PROMPT_ONLY mode, use standard content processing
         if mode == ProcessMode.PROMPT_ONLY:
+            messages = self._build_content_messages(block_index, variables)
             return LLMResult(prompt=messages[-1]["content"], metadata={"messages": messages})
 
+        # For COMPLETE and STREAM modes with LLM provider, use dynamic interaction check
+        # LLM will decide whether content needs to be converted to interaction block
+        if self._llm_provider:
+            block = self.get_block(block_index)
+            if block.block_type == BlockType.CONTENT:
+                return self._process_with_dynamic_check(block_index, mode, context, variables)
+
+        # Fallback: Build messages using standard content processing
+        messages = self._build_content_messages(block_index, variables)
+
         if mode == ProcessMode.COMPLETE:
             if not self._llm_provider:
                 raise ValueError(LLM_PROVIDER_REQUIRED_ERROR)
 
-            content = await self._llm_provider.complete(messages)
-            return LLMResult(content=content, prompt=messages[-1]["content"])
+            result = self._llm_provider.complete(messages)
+            return LLMResult(content=result.content, prompt=messages[-1]["content"], metadata=result.metadata)
 
         if mode == ProcessMode.STREAM:
             if not self._llm_provider:
                 raise ValueError(LLM_PROVIDER_REQUIRED_ERROR)
 
-            async def stream_generator():
-                async for chunk in self._llm_provider.stream(messages):  # type: ignore[attr-defined]
+            def stream_generator():
+                for chunk in self._llm_provider.stream(messages):
                     yield LLMResult(content=chunk, prompt=messages[-1]["content"])
 
             return stream_generator()
 
-    async def _process_preserved_content(self, block_index: int, variables: dict[str, str | list[str]] | None) -> LLMResult:
+    def _process_preserved_content(self, block_index: int, variables: dict[str, str | list[str]] | None) -> LLMResult:
         """Process preserved content block, output as-is without LLM call."""
         block = self.get_block(block_index)
 
@@ -265,7 +281,7 @@ class MarkdownFlow:
 
         return LLMResult(content=content)
 
-    async def _process_interaction_render(self, block_index: int, mode: ProcessMode, variables: dict[str, str | list[str]] | None = None) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    def _process_interaction_render(self, block_index: int, mode: ProcessMode, variables: dict[str, str | list[str]] | None = None) -> LLMResult | Generator[LLMResult, None, None]:
         """Process interaction content rendering."""
         block = self.get_block(block_index)
 
@@ -298,7 +314,8 @@ class MarkdownFlow:
             if not self._llm_provider:
                 return LLMResult(content=processed_block.content)  # Fallback processing
 
-            rendered_question = await self._llm_provider.complete(messages)
+            result = self._llm_provider.complete(messages)
+            rendered_question = result.content
             rendered_content = self._reconstruct_interaction_content(processed_block.content, rendered_question)
 
             return LLMResult(
@@ -315,7 +332,7 @@ class MarkdownFlow:
                 # For interaction blocks, return reconstructed content (one-time output)
                 rendered_content = self._reconstruct_interaction_content(processed_block.content, question_text or "")
 
-                async def stream_generator():
+                def stream_generator():
                     yield LLMResult(
                         content=rendered_content,
                         prompt=messages[-1]["content"],
@@ -324,9 +341,9 @@ class MarkdownFlow:
                 return stream_generator()
 
             # With LLM provider, collect full response then return once
-            async def stream_generator():
+            def stream_generator():
                 full_response = ""
-                async for chunk in self._llm_provider.stream(messages):  # type: ignore[attr-defined]
+                for chunk in self._llm_provider.stream(messages):
                     full_response += chunk
 
                 # Reconstruct final interaction content
@@ -340,22 +357,23 @@ class MarkdownFlow:
 
             return stream_generator()
 
-    async def _process_interaction_input(
+    def _process_interaction_input(
         self,
         block_index: int,
         user_input: dict[str, list[str]],
         mode: ProcessMode,
         context: list[dict[str, str]] | None,
         variables: dict[str, str | list[str]] | None = None,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """Process interaction user input."""
+        _ = context  # Mark as intentionally unused
         block = self.get_block(block_index)
         target_variable = block.variables[0] if block.variables else "user_input"
 
         # Basic validation
         if not user_input or not any(values for values in user_input.values()):
             error_msg = INPUT_EMPTY_ERROR
-            return await self._render_error(error_msg, mode)
+            return self._render_error(error_msg, mode)
 
         # Get the target variable value from user_input
         target_values = user_input.get(target_variable, [])
@@ -369,7 +387,7 @@ class MarkdownFlow:
 
         if "error" in parse_result:
             error_msg = INTERACTION_PARSE_ERROR.format(error=parse_result["error"])
-            return await self._render_error(error_msg, mode)
+            return self._render_error(error_msg, mode)
 
         interaction_type = parse_result.get("type")
 
@@ -381,7 +399,7 @@ class MarkdownFlow:
             InteractionType.BUTTONS_MULTI_WITH_TEXT,
         ]:
             # All button types: validate user input against available buttons
-            return await self._process_button_validation(
+            return self._process_button_validation(
                 parse_result,
                 target_values,
                 target_variable,
@@ -414,16 +432,16 @@ class MarkdownFlow:
                 },
             )
         error_msg = f"No input provided for variable '{target_variable}'"
-        return await self._render_error(error_msg, mode)
+        return self._render_error(error_msg, mode)
 
-    async def _process_button_validation(
+    def _process_button_validation(
         self,
         parse_result: dict[str, Any],
         target_values: list[str],
         target_variable: str,
         mode: ProcessMode,
         interaction_type: InteractionType,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """
         Simplified button validation with new input format.
 
@@ -458,7 +476,7 @@ class MarkdownFlow:
             # Pure button mode requires input
             button_displays = [btn["display"] for btn in buttons]
             error_msg = f"Please select from: {', '.join(button_displays)}"
-            return await self._render_error(error_msg, mode)
+            return self._render_error(error_msg, mode)
 
         # Validate input values against available buttons
         valid_values = []
@@ -474,7 +492,7 @@ class MarkdownFlow:
 
             if not matched:
                 if allow_text_input:
-                    # Allow custom text in buttons+text mode
+                    # Allow custom text in buttons+text mode - same as normal interactions
                     valid_values.append(value)
                 else:
                     invalid_values.append(value)
@@ -483,7 +501,7 @@ class MarkdownFlow:
         if invalid_values and not allow_text_input:
             button_displays = [btn["display"] for btn in buttons]
             error_msg = f"Invalid options: {', '.join(invalid_values)}. Please select from: {', '.join(button_displays)}"
-            return await self._render_error(error_msg, mode)
+            return self._render_error(error_msg, mode)
 
         # Success: return validated values
         return LLMResult(
@@ -498,13 +516,13 @@ class MarkdownFlow:
             },
         )
 
-    async def _process_llm_validation(
+    def _process_llm_validation(
         self,
         block_index: int,
         user_input: dict[str, list[str]],
         target_variable: str,
         mode: ProcessMode,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """Process LLM validation."""
         # Build validation messages
         messages = self._build_validation_messages(block_index, user_input, target_variable)
@@ -523,7 +541,8 @@ class MarkdownFlow:
                 # Fallback processing, return variables directly
                 return LLMResult(content="", variables=user_input)  # type: ignore[arg-type]
 
-            llm_response = await self._llm_provider.complete(messages)
+            result = self._llm_provider.complete(messages)
+            llm_response = result.content
 
             # Parse validation response and convert to LLMResult
             # Use joined target values for fallback; avoids JSON string injection
@@ -535,9 +554,9 @@ class MarkdownFlow:
             if not self._llm_provider:
                 return LLMResult(content="", variables=user_input)  # type: ignore[arg-type]
 
-            async def stream_generator():
+            def stream_generator():
                 full_response = ""
-                async for chunk in self._llm_provider.stream(messages):  # type: ignore[attr-defined]
+                for chunk in self._llm_provider.stream(messages):
                     full_response += chunk
 
                 # Parse complete response and convert to LLMResult
@@ -551,7 +570,7 @@ class MarkdownFlow:
 
             return stream_generator()
 
-    async def _process_llm_validation_with_options(
+    def _process_llm_validation_with_options(
         self,
         block_index: int,
         user_input: dict[str, list[str]],
@@ -559,8 +578,9 @@ class MarkdownFlow:
         options: list[str],
         question: str,
         mode: ProcessMode,
-    ) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    ) -> LLMResult | Generator[LLMResult, None, None]:
         """Process LLM validation with button options (third case)."""
+        _ = block_index  # Mark as intentionally unused
         # Build special validation messages containing button option information
         messages = self._build_validation_messages_with_options(user_input, target_variable, options, question)
 
@@ -580,7 +600,8 @@ class MarkdownFlow:
                 # Fallback processing, return variables directly
                 return LLMResult(content="", variables=user_input)  # type: ignore[arg-type]
 
-            llm_response = await self._llm_provider.complete(messages)
+            result = self._llm_provider.complete(messages)
+            llm_response = result.content
 
             # Parse validation response and convert to LLMResult
             # Use joined target values for fallback; avoids JSON string injection
@@ -592,9 +613,9 @@ class MarkdownFlow:
             if not self._llm_provider:
                 return LLMResult(content="", variables=user_input)  # type: ignore[arg-type]
 
-            async def stream_generator():
+            def stream_generator():
                 full_response = ""
-                async for chunk in self._llm_provider.stream(messages):  # type: ignore[attr-defined]
+                for chunk in self._llm_provider.stream(messages):
                     full_response += chunk
                     # For validation scenario, don't output chunks in real-time, only final result
 
@@ -611,7 +632,7 @@ class MarkdownFlow:
 
             return stream_generator()
 
-    async def _render_error(self, error_message: str, mode: ProcessMode) -> LLMResult | AsyncGenerator[LLMResult, None]:
+    def _render_error(self, error_message: str, mode: ProcessMode) -> LLMResult | Generator[LLMResult, None, None]:
         """Render user-friendly error message."""
         messages = self._build_error_render_messages(error_message)
 
@@ -625,15 +646,16 @@ class MarkdownFlow:
             if not self._llm_provider:
                 return LLMResult(content=error_message)  # Fallback processing
 
-            friendly_error = await self._llm_provider.complete(messages)
+            result = self._llm_provider.complete(messages)
+            friendly_error = result.content
             return LLMResult(content=friendly_error, prompt=messages[-1]["content"])
 
         if mode == ProcessMode.STREAM:
             if not self._llm_provider:
                 return LLMResult(content=error_message)
 
-            async def stream_generator():
-                async for chunk in self._llm_provider.stream(messages):  # type: ignore[attr-defined]
+            def stream_generator():
+                for chunk in self._llm_provider.stream(messages):
                     yield LLMResult(content=chunk, prompt=messages[-1]["content"])
 
             return stream_generator()
@@ -795,3 +817,351 @@ Original Error: {error_message}
             suffix = match.group(2)
             return f"{prefix}{cleaned_question}{suffix}"
         return original_content  # type: ignore[unreachable]
+
+    # Dynamic Interaction Methods
+
+    def _process_with_dynamic_check(
+        self,
+        block_index: int,
+        mode: ProcessMode,
+        context: list[dict[str, str]] | None,
+        variables: dict[str, str | list[str]] | None,
+    ) -> LLMResult | Generator[LLMResult, None, None]:
+        """Process content with dynamic interaction detection and conversion."""
+
+        block = self.get_block(block_index)
+        messages = self._build_dynamic_check_messages(block, context, variables)
+
+        # Define Function Calling tools with structured approach
+        tools = [
+            {
+                "type": "function",
+                "function": {
+                    "name": "create_interaction_block",
+                    "description": "Convert content to interaction block with structured data when it needs to collect user input",
+                    "parameters": {
+                        "type": "object",
+                        "properties": {
+                            "needs_interaction": {"type": "boolean", "description": "Whether this content needs to be converted to interaction block"},
+                            "variable_name": {"type": "string", "description": "Name of the variable to collect (without {{}} brackets)"},
+                            "interaction_type": {
+                                "type": "string",
+                                "enum": ["single_select", "multi_select", "text_input", "mixed"],
+                                "description": "Type of interaction: single_select (|), multi_select (||), text_input (...), mixed (options + text)",
+                            },
+                            "options": {"type": "array", "items": {"type": "string"}, "description": "List of selectable options (3-4 specific options based on context)"},
+                            "allow_text_input": {"type": "boolean", "description": "Whether to include a text input option for 'Other' cases"},
+                            "text_input_prompt": {"type": "string", "description": "Prompt text for the text input option (e.g., '其他请输入', 'Other, please specify')"},
+                        },
+                        "required": ["needs_interaction"],
+                    },
+                },
+            }
+        ]
+
+        if not self._llm_provider:
+            raise ValueError(LLM_PROVIDER_REQUIRED_ERROR)
+
+        # Call LLM with tools
+        result = self._llm_provider.complete(messages, tools)
+
+        # If interaction was generated through Function Calling, construct the MarkdownFlow format
+        if result.transformed_to_interaction and result.metadata and "tool_args" in result.metadata:
+            tool_args = result.metadata["tool_args"]
+            if tool_args.get("needs_interaction"):
+                # Construct MarkdownFlow format from structured data
+                interaction_content = self._build_interaction_format(tool_args)
+                result.content = interaction_content
+
+        # If transformed to interaction, return as is
+        if result.transformed_to_interaction:
+            return result
+
+        # If not transformed, continue with normal processing using standard content messages
+        normal_messages = self._build_content_messages(block_index, variables)
+
+        if mode == ProcessMode.STREAM:
+
+            def stream_wrapper():
+                stream_generator = self._llm_provider.stream(normal_messages)
+                for chunk in stream_generator:
+                    yield LLMResult(content=chunk)
+
+            return stream_wrapper()
+
+        # Complete mode - use normal content processing
+        normal_result = self._llm_provider.complete(normal_messages)
+        return LLMResult(content=normal_result.content, prompt=normal_messages[-1]["content"], metadata=normal_result.metadata)
+
+    def _build_dynamic_check_messages(
+        self,
+        block: "Block",
+        context: list[dict[str, str]] | None,
+        variables: dict[str, str | list[str]] | None,
+    ) -> list[dict[str, str]]:
+        """Build messages for dynamic interaction detection."""
+
+        import json
+
+        # System prompt for detection
+        system_prompt = """You are an intelligent document processing assistant specializing in creating interactive forms.
+
+Task: Analyze the given content block and determine if it needs to be converted to an interaction block to collect user information.
+
+Judgment criteria:
+1. Does the content imply the need to ask users for information?
+2. Does it need to collect detailed information based on previous variable values?
+3. Does it mention "recording" or "saving" information to variables?
+
+If conversion is needed, generate a STANDARD interaction block format with SPECIFIC options based on the document-level instructions and context provided in the user message."""
+
+        # User message with content and context
+        # Build user prompt with document context
+        user_prompt_parts = []
+
+        # Add document-level prompt context if exists
+        if self._document_prompt:
+            user_prompt_parts.append(f"""Document-level instructions:
+{self._document_prompt}
+
+(Note: The above are the user's document-level instructions that provide context and requirements for processing.)
+""")
+
+        # Prepare content analysis with both original and resolved versions
+        original_content = block.content
+
+        # Create resolved content with variable substitution for better context
+        resolved_content = original_content
+        if variables:
+            from .utils import replace_variables_in_text
+
+            resolved_content = replace_variables_in_text(original_content, variables)
+
+        content_analysis = f"""Current content block to analyze:
+
+**Original content (shows variable structure):**
+{original_content}
+
+**Resolved content (with current variable values):**
+{resolved_content}
+
+**Existing variable values:**
+{json.dumps(variables, ensure_ascii=False) if variables else "None"}"""
+
+        # Add different analysis based on whether content has variables
+        if "{{" in original_content and "}}" in original_content:
+            from .utils import extract_variables_from_text
+
+            content_variables = set(extract_variables_from_text(original_content))
+
+            # Find new variables (not yet collected)
+            new_variables = content_variables - (set(variables.keys()) if variables else set())
+            existing_used_variables = content_variables & (set(variables.keys()) if variables else set())
+
+            content_analysis += f"""
+
+**Variable analysis:**
+- Variables used from previous steps: {list(existing_used_variables) if existing_used_variables else "None"}
+- New variables to collect: {list(new_variables) if new_variables else "None"}
+
+**Context guidance:**
+- Use the resolved content to understand the actual context and requirements
+- Generate options based on the real variable values shown in the resolved content
+- Collect user input for the new variables identified above"""
+
+        user_prompt_parts.append(content_analysis)
+
+        # Add analysis requirements and structured output guide
+        user_prompt_parts.append("""## Analysis Task:
+1. Determine if this content needs to be converted to an interaction block
+2. If conversion is needed, provide structured interaction data
+
+## Context-based Analysis:
+- Use the "Resolved content" to understand actual context (e.g., if it shows "川菜", generate Sichuan dish options)
+- Extract the "New variables to collect" identified in the variable analysis above
+- Generate 3-4 specific options based on the resolved context and document-level instructions
+- Follow ALL document-level instruction requirements (language, domain, terminology)
+
+## Selection Type Decision Logic:
+Ask: "Can a user realistically want/choose multiple of these options simultaneously?"
+
+**Use MULTI_SELECT when:**
+- Food dishes (can order multiple: 宫保鸡丁, 麻婆豆腐)
+- Programming skills (can know multiple: Python, JavaScript)
+- Interests/hobbies (can have multiple: 读书, 运动, 旅游)
+- Product features (can want multiple: 定制颜色, 个性化logo)
+- Exercise types (can do multiple: 跑步, 游泳, 瑜伽)
+
+**Use SINGLE_SELECT when:**
+- Job positions (usually apply for one: 软件工程师 OR 产品经理)
+- Experience levels (have one current level: Beginner OR Advanced)
+- Budget ranges (have one range: 5-10万 OR 10-20万)
+- Education levels (have one highest: Bachelor's OR Master's)
+
+## Output Instructions:
+If this content needs interaction, use the create_interaction_block function with:
+- `needs_interaction`: true/false
+- `variable_name`: the variable to collect (from "New variables" above)
+- `interaction_type`: "single_select", "multi_select", "text_input", or "mixed"
+- `options`: array of 3-4 specific options based on context
+- `allow_text_input`: true if you want to include "other" option
+- `text_input_prompt`: text for the "other" option (in appropriate language)
+
+Analyze the content and provide the structured interaction data.""")
+
+        user_prompt = "\n\n".join(user_prompt_parts)
+
+        messages = [{"role": "system", "content": system_prompt}]
+
+        # Add context if provided
+        if context:
+            messages.extend(context)
+
+        messages.append({"role": "user", "content": user_prompt})
+
+        return messages
+
+    def _build_interaction_format(self, tool_args: dict) -> str:
+        """Build MarkdownFlow interaction format from structured Function Calling data."""
+        variable_name = tool_args.get("variable_name", "")
+        interaction_type = tool_args.get("interaction_type", "single_select")
+        options = tool_args.get("options", [])
+        allow_text_input = tool_args.get("allow_text_input", False)
+        text_input_prompt = tool_args.get("text_input_prompt", "...请输入")
+
+        if not variable_name:
+            return ""
+
+        # For text_input type, options can be empty
+        if interaction_type != "text_input" and not options:
+            return ""
+
+        # Choose separator based on interaction type
+        if interaction_type in ["multi_select", "mixed"]:
+            separator = "||"
+        else:
+            separator = "|"
+
+        # Build options string
+        if interaction_type == "text_input":
+            # Text input only
+            options_str = f"...{text_input_prompt}"
+        else:
+            # Options with potential text input
+            options_str = separator.join(options)
+
+            if allow_text_input and text_input_prompt:
+                # Ensure text input has ... prefix
+                text_option = text_input_prompt if text_input_prompt.startswith("...") else f"...{text_input_prompt}"
+                options_str += f"{separator}{text_option}"
+
+        return f"?[%{{{{{variable_name}}}}} {options_str}]"
+
+    def _process_dynamic_interaction_validation(
+        self,
+        block_index: int,
+        interaction_format: str,
+        user_input: dict[str, list[str]],
+        mode: ProcessMode,
+        context: list[dict[str, str]] | None,
+        variables: dict[str, str | list[str]] | None,
+    ) -> LLMResult:
+        """Validate user input for dynamically generated interaction blocks using same logic as normal interactions."""
+        _ = block_index  # Mark as intentionally unused
+        _ = context  # Mark as intentionally unused
+
+        from .utils import InteractionParser
+
+        # Parse the interaction format using the same parser as normal interactions
+        parser = InteractionParser()
+        parse_result = parser.parse(interaction_format)
+
+        if "error" in parse_result:
+            error_msg = f"Invalid interaction format: {parse_result['error']}"
+            return self._render_error(error_msg, mode)
+
+        # Extract variable name and interaction type
+        variable_name = parse_result.get("variable")
+        interaction_type = parse_result.get("type")
+
+        if not variable_name:
+            error_msg = f"No variable found in interaction format: {interaction_format}"
+            return self._render_error(error_msg, mode)
+
+        # Get user input for the target variable
+        target_values = user_input.get(variable_name, [])
+
+        # Basic validation - check if input is provided when required
+        if not target_values:
+            # Check if this is a text input or allows empty input
+            allow_text_input = interaction_type in [
+                InteractionType.BUTTONS_WITH_TEXT,
+                InteractionType.BUTTONS_MULTI_WITH_TEXT,
+            ]
+
+            if allow_text_input:
+                # Allow empty input for buttons+text mode - merge with existing variables
+                merged_variables = dict(variables or {})
+                merged_variables[variable_name] = []
+                return LLMResult(
+                    content="",
+                    variables=merged_variables,
+                    metadata={
+                        "interaction_type": "dynamic_interaction",
+                        "empty_input": True,
+                    },
+                )
+            else:
+                error_msg = f"No input provided for variable '{variable_name}'"
+                return self._render_error(error_msg, mode)
+
+        # Use the same validation logic as normal interactions
+        if interaction_type in [
+            InteractionType.BUTTONS_ONLY,
+            InteractionType.BUTTONS_WITH_TEXT,
+            InteractionType.BUTTONS_MULTI_SELECT,
+            InteractionType.BUTTONS_MULTI_WITH_TEXT,
+        ]:
+            # Button validation - reuse the existing button validation logic
+            button_result = self._process_button_validation(
+                parse_result,
+                target_values,
+                variable_name,
+                mode,
+                interaction_type,
+            )
+
+            # Merge with existing variables for dynamic interactions
+            if hasattr(button_result, 'variables') and button_result.variables is not None and variables:
+                merged_variables = dict(variables)
+                merged_variables.update(button_result.variables)
+                return LLMResult(
+                    content=button_result.content,
+                    variables=merged_variables,
+                    metadata=button_result.metadata,
+                )
+            return button_result
+
+        elif interaction_type == InteractionType.NON_ASSIGNMENT_BUTTON:
+            # Non-assignment buttons: don't set variables, keep existing ones
+            return LLMResult(
+                content="",
+                variables=dict(variables or {}),
+                metadata={
+                    "interaction_type": "non_assignment_button",
+                    "user_input": user_input,
+                },
+            )
+        else:
+            # Text-only input type - merge with existing variables
+            merged_variables = dict(variables or {})
+            merged_variables[variable_name] = target_values
+            return LLMResult(
+                content="",
+                variables=merged_variables,
+                metadata={
+                    "interaction_type": "text_only",
+                    "target_variable": variable_name,
+                    "values": target_values,
+                },
+            )
